package it.arnaldo.unibs.tamagolem;

import java.util.*;

public class WorldBuilder {
    public static ElementGraph buildWorld() {
        Element[] allElements = Element.values();
        Random rand = new Random();
        // da cambiare il numero a 8
        int numElements = 3 + rand.nextInt(3);
        List<Element> selectedElements = new ArrayList<>(Arrays.asList(allElements));
        Collections.shuffle(selectedElements);
        selectedElements = selectedElements.subList(0, numElements);

        ElementGraph world = new ElementGraph();

        HashMap<Element, Integer> elementStrengthNumber = new HashMap<>();
        HashMap<Element, Integer> elementStrengthSum = new HashMap<>();
        HashMap<Element, Integer> elementWeaknessPointToAssign = new HashMap<>();
        HashMap<Element, Integer> elementWeaknessPoint = new HashMap<>();

        int numMaxOfPossibleStrength = (numElements * (numElements - 1) / 2) - numElements;
        System.out.println(numMaxOfPossibleStrength);
        int index = 0;
        for (Element element : selectedElements) {
            index++;
            int randomNumber = rand.nextInt(numElements - 2);
            if (numMaxOfPossibleStrength - randomNumber < 0) {
                randomNumber = numMaxOfPossibleStrength;
                numMaxOfPossibleStrength = 0;
            } else {
                numMaxOfPossibleStrength -= randomNumber;
            }

            if (index == numElements) {
                randomNumber = numMaxOfPossibleStrength;
            }
            int strengthNumber = 1 + randomNumber;
            int weaknessNumber = numElements - 1 - strengthNumber;
            // int strengthSum = Math.max(strengthNumber, weaknessNumber) + rand.nextInt(10);
            elementStrengthNumber.put(element, strengthNumber);
            elementWeaknessPoint.put(element, weaknessNumber);
        }


        Map<Element, GraphElement> graphElementMap = new HashMap<>();
        for (Element el : selectedElements) {
            GraphElement ge = new GraphElement(el);
            world.addElement(ge);
            graphElementMap.put(el, ge);
        }

        // Pairing e bilanciamento
        List<Element[]> possibleInteractions = new ArrayList<>();
        for (Element attacker : selectedElements) {
            for (Element defender : selectedElements) {
                if (!attacker.equals(defender)) {
                    possibleInteractions.add(new Element[]{attacker, defender});
                }
            }
        }

        Collections.shuffle(possibleInteractions);

        Set<String> alreadyUsed = new HashSet<>();

        for (Element[] pair : possibleInteractions) {
            Element attacker = pair[0];
            Element defender = pair[1];

            String key = attacker.name() + "-" + defender.name();
            String reverseKey = defender.name() + "-" + attacker.name();
            if (alreadyUsed.contains(key) || alreadyUsed.contains(reverseKey)) continue;

            int attackerRemaining = elementStrengthNumber.get(attacker);
            int defenderRemaining = elementWeaknessPoint.get(defender);

            int attackerPoints = elementStrengthSum.get(attacker);
            int defenderPoints = elementWeaknessPointToAssign.get(defender);

            if (attackerRemaining <= 0 || defenderRemaining <= 0) continue;

            // Calcolo potenza massima assegnabile
            int maxPower = Math.min(attackerPoints - (attackerRemaining - 1), defenderPoints - (defenderRemaining - 1));
            if (maxPower < 1) continue;

            int power = 1 + rand.nextInt(maxPower);

            // Assegna collegamenti
            graphElementMap.get(attacker).addLink(new ElementLink(power, defender));
            graphElementMap.get(defender).addLink(new ElementLink(-power, attacker));
            alreadyUsed.add(key);
            alreadyUsed.add(reverseKey);

            // Aggiorna contatori
            elementStrengthNumber.put(attacker, attackerRemaining - 1);
            elementStrengthSum.put(attacker, attackerPoints - power);

            elementWeaknessPoint.put(defender, defenderRemaining - 1);
            elementWeaknessPointToAssign.put(defender, defenderPoints - power);
        }

        System.out.println(numElements);

        world.printGraph();

        return world;
    }

}
